<?php

/**
 * @file
 * Contains \Drupal\lockr\Form\LockrAdminForm.
 */

namespace Drupal\lockr\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\State\StateInterface;

use Egulias\EmailValidator\EmailValidator;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\RequestException;
use Lockr\Exception\LockrClientException;
use Lockr\Exception\LockrException;
use Symfony\Component\DependencyInjection\ContainerInterface;

use Drupal\lockr\LockrClientInterface;
use Drupal\lockr\LockrPartnerInterface;

class LockrAdminForm extends FormBase {

  /**
   * @var LockrClientInterface
   */
  protected $client;

  protected $certRequestUrl = 'https://lockr.io/api/v2/request-certificate';

  public function setCertRequestUrl($url) {
    $this->certRequestUrl = $url;
  }

  /**
   * Constructs a new LockrAdminForm.
   *
   * @param LockrClientInterface $client
   */
  public function __construct(
    StateInterface $state,
    ConfigFactoryInterface $config,
    LockrClientInterface $client,
    LockrPartnerInterface $partner,
    EmailValidator $emailValidator,
    ClientInterface $httpClient,
    $drupalRoot
  ) {
    $this->state = $state;
    $this->config = $config;
    $this->client = $client;
    $this->partner = $partner;
    $this->emailValidator = $emailValidator;
    $this->httpClient = $httpClient;
    $this->drupalRoot = $drupalRoot;
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('state'),
      $container->get('config.factory'),
      $container->get('lockr.client'),
      $container->get('lockr.partner'),
      $container->get('email.validator'),
      $container->get('http_client'),
      $container->get('app.root')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'lockr_admin_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    try {
      list($exists) = $this->client->exists();
    }
    catch (LockrException $e) {
      $form['trouble'] = [
        '#prefix' => '<p>',
        '#markup' => $this->t('A connection to Lockr could not be established, please contact support@lockr.io for assistance.'),
        '#suffix' => '</p>',
      ];
      return $form;
    }

    $partner = $this->partner->getPartner();
    $request_sent = (bool) $this->state->get('lockr.request', FALSE);

    $register_form = (!$exists && ($partner !== NULL)) || $request_sent;
    $advanced_open = !$exists && !$partner;
    $request_form = !$exists && $partner === NULL && !$request_sent;

    if ($partner !== NULL) {
      $desc = $this->t('Our system has detected that your website is hosted on one of our supported providers.');
    }
    elseif ($request_sent) {
      $desc = $this->t('Thank you for your request for a custom cert! A member from our development team will be generating that cert and getting back to you shortly.');
    }
    else {
      $desc = $this->t('Thank you for your interest in Lockr! Our system is detecting that your website is not currently hosted by a supported provider. To help us better prioritize our integrations, please provide your current hosting provider and email in the fields below. We will be sure to notify you when Lockr is integrated with your hosting provider. In the meantime, a custom certificate to use Lockr will be generated and sent to the email you provided.');
    }

    $form['description'] = [
      '#prefix' => '<p>',
      '#plain_text' => $desc,
      '#suffix' => '</p>',
    ];

    if ($register_form) {
      $form['register'] = $this->buildRegisterForm();
    }
    elseif ($exists) {
      $form['registered'] = [
        '#title' => $this->t("You're already registered"),
        '#prefix' => '<p>',
        '#markup' => $this->t("This site is already registered with the Lockr Key Management Service. There's nothing left for you do to here, your keys entered in the key settings are already protected. If you registered with the wrong account, you can click <a href=\"https://lockr.io/user/login\" target=\"_blank\">here</a> to go to Lockr and manage your sites."),
        '#suffix' => '</p>',
      ];
    }

    if ($request_form) {
      $form['request'] = $this->buildRequestForm();
    }
    elseif ($request_sent) {
      $form['resend_request'] = [
        '#type' => 'submit',
        '#value' => $this->t('Resent request link'),
        '#submit' => [[$this, 'submitResentRequest']],
      ];
    }

    $form['advanced'] = $this->buildAdvancedForm($advanced_open);

    return $form;
  }

  public function buildRequestForm() {
    $form = [];

    $form['email'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Email address'),
    ];

    $form['partner'] = [
      '#type' => 'submit',
      '#value' => $this->t('Request certificate'),
      '#validate' => [[$this, 'validateRequestForm']],
      '#submit' => [[$this, 'submitRequestForm']],
    ];

    return $form;
  }

  public function validateRequestForm(
    array &$form,
    FormStateInterface $form_state
  ) {
    $email = $form_state->getValue('email');
    if (!$this->emailValidator->isValid($email)) {
      $form_state->setErrorByName(
        'email',
        $this->t('Please enter a valid email address.')
      );
    }
  }

  public function submitRequestForm(
    array &$form,
    FormStateInterface $form_state
  ) {
    $email = $form_state->getValue('email');
    $partner = $form_state->getValue('partner');
    try {
      $this->httpClient->post($this->certRequestUrl, [
        'form_params' => ['email' => $email, 'partner' => $partner],
      ]);
    }
    catch (RequestException $e) {
      drupal_set_message($this->t('An unknown error occurred trying to request certificate. Please try again later.'));
      return;
    }

    $this->state->set('lockr.request', TRUE);
    drupal_set_message($this->t("Thank you for letting us know! We'll email you shortly with further instructions."));
  }

  public function buildRegisterForm() {
    $form = [];

    $form['email'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Email address'),
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Sign up'),
      '#validate' => [[$this, 'validateRegisterForm']],
      '#submit' => [[$this, 'submitRegisterForm']],
    ];

    return $form;
  }

  public function validateRegisterForm(
    array &$form,
    FormStateInterface $form_state
  ) {
    $email = $form_state->getValue('email');
    if (!$this->emailValidator->isValid($email)) {
      $form_state->setErrorByName(
        'email',
        $this->t('Please enter a valid email address.')
      );
    }
  }

  public function submitRegisterForm(
    array &$form,
    FormStateInterface $form_state
  ) {
    $email = $form_state->getValue('email');
    $name = $this->getSiteName();

    try {
      $this->client->register($email, NULL, $name);
    }
    catch (LockrClientException $e) {
      if (
        $e->title === 'Missing header value' &&
        $e->description === 'The Auth header is required.'
      ) {
        $form_state->setRedirect('lockr.login');
        return;
      }
      elseif ($e->title === 'Partner mismatch') {
        $msg = $this->t("We didn't recognize your certificate, please ensure the provide path is a valid Lockr certificate.");
      }
      elseif ($e->title === 'Site exists') {
        $msg = $this->t('This site is already registered. If you are experiencing issues, please contact support@lockr.io.');
      }
      else {
        $msg = $this->t('An unknown error occurred, please try again later.');
      }

      drupal_set_message($msg, 'error');

      return;
    }
    catch (LockrException $e) {
      $msg = $this->t('An unknown error occurred, please try again later.');

      drupal_set_message($msg, 'error');

      return;
    }

    drupal_set_message($this->t("That's it! You're signed up with Lockr; your keys are now safe."));
    $form_state->setRedirect('entity.key.collection');
  }

  public function buildAdvancedForm($advanced_open) {
    $form = [
      '#type' => 'fieldset',
      '#title' => $this->t('Advanced'),
      '#collapsible' => TRUE,
      '#collapsed' => !$advanced_open,
    ];

    $form['region'] = [
      '#type' => 'radios',
      '#title' => $this->t('Region'),
      '#default_value' => $this->state->get('lockr.region', 'us'),
      '#options' => [
        'us' => $this->t('US'),
        'eu' => $this->t('EU'),
      ],
    ];

    $form['custom'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Set custom certificate location'),
      '#default_value' => (bool) $this->state->get('lockr.custom', FALSE),
    ];

    $form['custom_cert'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Certificate path'),
      '#default_value' => $this->state->get('lockr.cert', ''),
      '#states' => [
        'visible' => [':input[name="custom"]' => ['checked' => TRUE]],
      ],
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
      '#validate' => [[$this, 'validateAdvancedForm']],
      '#submit' => [[$this, 'submitAdvancedForm']],
    ];

    return $form;
  }

  public function validateAdvancedForm(
    array &$form,
    FormStateInterface $form_state
  ) {
    if (!$form_state->getValue('custom')) {
      return;
    }

    $cert_path = $form_state->getValue('custom_cert');

    if (!$cert_path) {
      $form_state->setErrorByName(
        'custom_cert',
        $this->t('Certificate location is required for custom certs')
      );
      return;
    }

    if ($cert_path[0] !== '/') {
      $cert_path = "{$this->drupalRoot}/{$cert_path}";
    }

    if (is_dir($cert_path) || !is_readable($cert_path)) {
      $form_state->setErrorByName(
        'custom_cert',
        $this->t('Certificate must be a readable file')
      );
    }
  }

  public function submitAdvancedForm(
    array &$form,
    FormStateInterface $form_state
  ) {
    $custom = $form_state->getValue('custom');
    $region = $form_state->getValue('region');

    $this->state->set('lockr.region', $region);

    $this->state->set('lockr.custom', $custom);
    if ($custom) {
      $this->state->setMultiple([
        'lockr.partner' => 'custom',
        'lockr.cert' => $form_state->getValue('custom_cert'),
      ]);
    }
    else {
      $this->state->deleteMultiple(['lockr.partner', 'lockr.cert']);
    }
  }

  protected function getSiteName() {
    return $this->config->get('system.site')->get('name');
  }

  protected function submitResentRequest(
    array &$form,
    FormStateInterface $form_state
  ) {
    $this->state->delete('lockr.request');
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
  }

}
